<?php
require_once(ABSPATH . 'wp-admin/includes/media.php');
require_once(ABSPATH . 'wp-admin/includes/file.php');
require_once(ABSPATH . 'wp-admin/includes/image.php');

class Focus_Keyword_Finder_Admin
{
    private $focus_keyword_finder;

    private $version;

    private $name;

    private $route;

    protected $base_url;

    protected $plugin_update_url;

    private $access_key_header_variable;

    public function __construct($focus_keyword_finder, $version, $name, $route, $access_key_header_variable, $plugin_update_url, $base_url)
    {
        $this->focus_keyword_finder = $focus_keyword_finder;
        $this->version = $version;
        $this->name = $name;
        $this->route = $route;
        $this->access_key_header_variable = $access_key_header_variable;
        $this->base_url = $base_url;
        $this->plugin_update_url = $plugin_update_url;
    }

    public function admin_menu()
    {
        add_submenu_page(
            "options-general.php",
            $this->name,
            $this->name,
            'manage_options',
            $this->focus_keyword_finder . '_settings',
            array($this, 'general_settings_page'),
            50
        );
    }

    public function allow_specific_html_tags($tags)
    {
        $tags['iframe'] = array('src' => true);
        return $tags;
    }

    public function plugin_row_meta($plugin_meta, $plugin_file)
    {
        if ($plugin_file === $this->focus_keyword_finder.'/'.$this->focus_keyword_finder.'.php') {
            $settings_page_url = admin_url('admin.php?page=' . $this->focus_keyword_finder . '_settings');
            $custom_link = '<a href="'. $settings_page_url .'" target="_blank">' . __('Update API Access Key', "focus-keyword-finder") . '</a>';
            $plugin_meta[] = $custom_link;
        }
        return $plugin_meta;
    }

    public function general_settings_page()
    {
        ?>
            <div class="card">
                <a href="https://focuskeywordfinder.com" target="_blank" alt="<?php echo $this->name; ?>" style="margin-top: 10px; display: block; text-align: center;">
                    <img src="<?php echo plugin_dir_url(__FILE__) . 'images/focus-keyword-finder.svg';  ?>" width="250"/>
                </a>
                <h1 class="title" style="display: none;"><?php echo get_admin_page_title(); ?></h1>
                <p><?php _e('Welcome to the official <strong>Focus Keyword Finder</strong> Wordpress plugin!', "focus-keyword-finder"); ?></p>
                <p><?php _e('This plugin establishes the connection between your Wordpress website and your <a href="https://focuskeywordfinder.com/api/login" target="_blank">Focus Keyword Finder account</a>, and makes it easier to create and update posts.', "focus-keyword-finder"); ?></p>
                <form method="post" action="options.php">
                    <?php settings_fields($this->focus_keyword_finder.'_settings_general'); ?>
                    <?php do_settings_sections($this->focus_keyword_finder.'_settings_general'); ?>
                    <!-- API Key -->
                    <fieldset>
                        <label for="<?=$this->focus_keyword_finder;?>_access_key"><h2><?php _e('API Access Key', "focus-keyword-finder"); ?></h2></label>
                        <input type="text" placeholder="<?php _e('Please insert your API access key', "focus-keyword-finder"); ?>" name="<?=$this->focus_keyword_finder;?>_access_key" value="<?php echo esc_attr(get_option($this->focus_keyword_finder.'_access_key')); ?>" class="large-text">
                    </fieldset>
                    <br>
                    <!-- Disable API Access -->
                    <fieldset>
                        <legend class="screen-reader-text"><span><?php _e('Enable API Access', "focus-keyword-finder"); ?></span></legend>
                        <label for="<?=$this->focus_keyword_finder;?>_enabled_api_key">
                            <input type="checkbox" name="<?=$this->focus_keyword_finder;?>_enabled_api_key" value="1" <?php checked(1, get_option($this->focus_keyword_finder.'_enabled_api_key'), true); ?> />
                            <span><?php _e('Enable API Access', "focus-keyword-finder"); ?></span>
                        </label>
                    </fieldset>
                    <!-- Submit -->
                    <?php submit_button(); ?>
                </form>
            </div>
        <?php
    }

    public function admin_init()
    {
        register_setting($this->focus_keyword_finder.'_settings_security', $this->focus_keyword_finder.'_active_change_wp_admin');

        register_setting($this->focus_keyword_finder.'_settings_general', $this->focus_keyword_finder.'_access_key');
        register_setting($this->focus_keyword_finder.'_settings_general', $this->focus_keyword_finder.'_enabled_api_key');
        register_setting($this->focus_keyword_finder.'_settings_general', $this->focus_keyword_finder.'_send_post_update');

        if (!get_option($this->focus_keyword_finder.'_send_post_update')) {
            update_option($this->focus_keyword_finder.'_send_post_update', 'ativado');
        }
    }

    public function rest_api_init()
    {
        register_rest_route($this->route, '/get_authors', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_authors'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/get_categories', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_categories'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/get_types', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_types'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/get_plugin', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_plugin'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/get_post', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_post'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/get_post_complete', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_post_complete'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/search_posts', array(
            'methods' => 'GET',
            'callback' => array($this, 'search_posts'),
            'permission_callback' => array($this, 'check_permissions'),
        ));

        register_rest_route($this->route, '/create_post', array(
            'methods' => 'POST',
            'callback' => array($this, 'create_post'),
            'permission_callback' => array($this, 'check_permissions'),
        ));
    }

    public function get_authors()
    {
        $users = get_users(array(
          'orderby' => 'name',
          'order' => 'ASC',
        ));
        $data = array();
        foreach ($users as $user) {
            $data[] = array(
              'id' => $user->ID,
              'name' => $user->display_name,
              'email' => $user->user_email
            );
        }
        return new WP_REST_Response($data, 200);
    }

    public function search_posts($request)
    {
        $search = $request->get_param('search');
        $type = $request->get_param('type');

        if (! $search) {
            return new WP_REST_Response(new WP_Error('invalid_param', 'Missing "search" parameter', array( 'status' => 400 )), 400);
        }

        if (! $type) {
            return new WP_REST_Response(new WP_Error('invalid_param', 'Missing "type" parameter', array( 'status' => 400 )), 400);
        }

        $args = array(
          'post_type' => $type,
          's' => $search,
          'post_status' => "any"
        );

        $query = new WP_Query($args);

        $data = array();
        if ($query->have_posts()) {
            while ($query->have_posts()) {
                $query->the_post();
                $data[] = array(
                  'id' => get_the_ID(),
                  'title' => get_the_title(),
                );
            }
            wp_reset_postdata();
        }

        return new WP_REST_Response($data, 200);
    }

    public function get_post($request)
    {
        $id = $request->get_param('id');

        if (! $id) {
            return new WP_REST_Response(new WP_Error('invalid_param', 'Missing "id" parameter', array( 'status' => 400 )), 400);
        }

        $post = get_post($id);

        if ($post) {
            $data = array(
              'id' => $post->ID,
              'name' => $post->post_title,
              'url' => $post->post_name,
              'type' => $post->post_type,
              'author' => (int) $post->post_author,
              'status' => $post->post_status,
              'categories' => $post->post_category,
            );
            return new WP_REST_Response(array("post" => $data), 200);
        } else {
            return new WP_REST_Response(array("post" => null), 200);
        }
    }

    public function get_post_complete($request)
    {
        $id = $request->get_param('id');

        if (! $id) {
            return new WP_REST_Response(new WP_Error('invalid_param', 'Missing "id" parameter', array( 'status' => 400 )), 400);
        }

        $post = get_post($id);

        if ($post) {
            $data = array(
              'id' => $post->ID,
              'name' => $post->post_title,
              'url' => $post->post_name,
              'type' => $post->post_type,
              'author' => (int) $post->post_author,
              'status' => $post->post_status,
              'description' => $this->SEOPluginAction("get_description", $id),
              'categories' => $post->post_category,
              'content' => $post->post_content,
              'image' => has_post_thumbnail($post->ID) ? get_the_post_thumbnail_url($post->ID) : null,
            );
            return new WP_REST_Response(array("post" => $data), 200);
        } else {
            return new WP_REST_Response(array("post" => null), 200);
        }
    }

    public function get_categories()
    {
        $args = array("hide_empty" => false);

        $categories = get_categories($args);

        $data = array();
        foreach ($categories as $category) {
            $data[] = array(
              'id' => $category->term_id,
              'name' => $category->name,
              'slug' => $category->slug,
              'count' => $category->count,
              'parent' => $category->parent,
            );
        }

        return new WP_REST_Response($data, 200);
    }

    public function get_types()
    {
        $args = array( 'public' => true);
        $post_types = get_post_types($args, 'objects');
        $data = array();
        if ($post_types) {
            foreach ($post_types as $post_type) {
                if ($post_type->name !== 'attachment') {
                    // Anything but "attachment"
                    $data[] = array(
                      'name' => $post_type->name,
                      'label' => $post_type->labels->singular_name,
                    );
                }
            }
        }
        return new WP_REST_Response($data, 200);
    }

    public function get_plugin()
    {
        return new WP_REST_Response(array("plugin" => $this->SEOPluginAction("name")), 200);
    }

    public function create_post($request)
    {
        $id = $request->get_param('id');
        $title = $request->get_param('title');
        $type = $request->get_param('type');
        $content = $request->get_param('content');
        $description = $request->get_param('description');
        $status = $request->get_param('status');
        $url = $request->get_param('url');
        $author = $request->get_param('author');
        $image = $request->get_param('image');
        $categories = $request->get_param('categories');

        // --------------------------------------------

        // Create or Update?
        $isCreatingPost = $request->has_param('id') ? false : true;

        if(!$isCreatingPost) {
            // Update
            $post = get_post($id);
            if(!$post) {
                return new WP_REST_Response(new WP_Error('post_not_found', 'Post was not found', array( 'status' => 400 )), 400);
            }
        }

        if(count($request->get_params()) < 2) {
            return new WP_REST_Response(new WP_Error('no_update_data', 'Please provide data to update', array( 'status' => 400 )), 400);
        }

        // Type
        if ($isCreatingPost) {
            // Only let the user choose "post_type" when creating, NOT when updating
            if (post_type_exists($type)) {
                $post_data['post_type'] = $type;
            } else {
                return new WP_REST_Response(new WP_Error('invalid_post_type', 'Please a valid post type', array( 'status' => 400 )), 400);
            }
        }

        // Title
        if ($title) {
            $post_data['post_title'] = wp_strip_all_tags($title);
        }

        // Content
        if ($content) {
            // Update images inside content
            $post_data['post_content'] = $this->upload_images_and_replace_content($content);
        }

        // Status
        if ($status) {
            $valid_statuses = array( 'publish', 'draft' );
            if (! in_array($status, $valid_statuses)) {
                return new WP_REST_Response(new WP_Error('invalid_status', 'Invalid status provided. Use "publish" or "draft"', array( 'status' => 400 )), 400);
            }
            $post_data['post_status'] = $status;
        }

        // URL
        if ($url) {
            $post_data['post_name'] = sanitize_title($url);
        }

        // Author
        if ($author) {
            $user = get_user_by('id', $author);
            if ($user && $user->exists()) {
                $post_data['post_author'] = $author;
            } else {
                return new WP_REST_Response(new WP_Error('author_not_found', 'The provided author ID does not exist', array( 'status' => 400 )), 400);
            }
        }

        $attachment_id = null;

        // Upload image
        if ($image) {
            $attachment_id = media_sideload_image($image, 0, null, "id");
            if (is_wp_error($attachment_id)) {
                return new WP_REST_Response(new WP_Error('featured_media_failed', 'Failed to upload featured media: ' . $attachment_id->get_error_message()), 400);
            }
        }

        // Create or Update?
        if($isCreatingPost) {
            $id = wp_insert_post($post_data);
        } else {
            $post_data['ID'] = $id;
            $id = wp_update_post($post_data);
        }

        // Add uploaded image as thumbnail
        if ($attachment_id) {
            set_post_thumbnail($id, $attachment_id);
        }

        // Add categories
        if ($categories) {
            wp_set_post_categories($id, json_decode($categories, true));
        }

        // Update the Meta description
        if ($description) {
            $this->SEOPluginAction("set_description", $id, $description);
        }

        if ($id) {
            return new WP_REST_Response(array("id" => $id, "url" => get_permalink($id)), 200);
        } else {
            if($isCreatingPost) {
                return new WP_REST_Response(new WP_Error('post_create_fail', 'Failed to create the post', array( 'status' => 500 )), 500);
            } else {
                return new WP_REST_Response(new WP_Error('post_update_fail', 'Failed to update the post', array( 'status' => 500 )), 500);
            }
        }
    }

    public function check_permissions($request)
    {
        $server_access_key = get_option($this->focus_keyword_finder.'_access_key');
        $client_access_key = $request->get_header($this->access_key_header_variable);
        $is_access_enabled = get_option($this->focus_keyword_finder.'_enabled_api_key');

        if($is_access_enabled !== "1") {
            // API access is disabled
            return false;
        }

        if($server_access_key !== $client_access_key) {
            // Invalid API access key
            return false;
        }

        return true;
    }

    public function SEOPluginAction($info, $id = null, $update_data = null)
    {
        switch (true) {
            case function_exists('wpseo_init'):
                // ::::::::::::::::::: YOAST SEO
                switch ($info) {
                    case 'name':
                        return "YoastSEO";
                        break;
                    case 'get_description':
                        return get_post_meta($id, '_yoast_wpseo_metadesc', true);
                        break;
                    case 'set_description':
                        update_post_meta($id, '_yoast_wpseo_metadesc', $update_data);
                        break;
                }
                break;

            case function_exists('rank_math'):
                // ::::::::::::::::::: RANK MATH
                switch ($info) {
                    case 'name':
                        return "Rank Math";
                        break;
                    case 'get_description':
                        return get_post_meta($id, 'rank_math_description', true);
                        break;
                    case 'set_description':
                        update_post_meta($id, 'rank_math_description', $update_data);
                        break;
                }
                break;

            case function_exists('aioseo'):
                // ::::::::::::::::::: ALL IN ONE SEO
                switch ($info) {
                    case 'name':
                        return "All in One SEO";
                        break;
                    case 'get_description':
                        return get_post_meta($id, '_aioseo_description', true);
                        break;
                    case 'set_description':
                        update_post_meta($id, '_aioseo_description', $update_data);
                        break;
                }
                break;
        }

        // Defaults to null
        return null;
    }

    public function upload_images_and_replace_content($content)
    {
        preg_match_all('/<img\s+[^>]*src\s*=\s*(["\'])([^"\']+)\1[^>]*>/i', $content, $matches);
        $image_urls = array_unique($matches[2]);
        if (!empty($image_urls)) {
            foreach ($image_urls as $image_url) {
                // Only upload real external images (not the ones from current domain)
                if($this->is_external_url($image_url)) {
                    $new_image_url = media_sideload_image($image_url, 0, null, "src");
                    if($new_image_url) {
                        // Replace image on the $content
                        $content = str_replace($image_url, $new_image_url, $content);
                    }
                }
            }
        }
        return $content;
    }

    public function is_external_url($image_url)
    {
        $current_domain = get_site_url();
        $parsed_url = parse_url($image_url);
        $image_domain = isset($parsed_url['host']) ? $parsed_url['host'] : null;
        return $image_domain && $image_domain !== $current_domain;
    }

    public function update_plugin($update, array $plugin_data, string $plugin_file)
    {
        if ($plugin_file !== $this->focus_keyword_finder.'/'.$this->focus_keyword_finder.'.php') {
            return $update;
        }
        if (! empty($update)) {
            return $update;
        }
        $changelog = $this->plugin_request();
        if (! version_compare($plugin_data['Version'], $changelog->latest_version, '<')) {
            return $update;
        }
        return [
            'slug' => $this->focus_keyword_finder,
            'version' => $changelog->latest_version,
            'url' => $plugin_data['PluginURI'],
            'package' => $changelog->download_url,
        ];
    }

    public function plugin_request()
    {
        $access = wp_remote_get($this->plugin_update_url, array( 'timeout' => 10, 'headers' => array( 'Accept' => 'application/json' ) ));
        if (! is_wp_error($access) && 200 === wp_remote_retrieve_response_code($access)) {
            $result = json_decode(wp_remote_retrieve_body($access));
            return $result;
        }
    }
}
