<?php

class Focus_Keyword_Finder
{
    protected $loader;

    protected $focus_keyword_finder;

    protected $version;

    protected $name;

    protected $access_key_header_variable;

    protected $base_url;

    protected $plugin_update_url;

    protected $route;

    public function __construct()
    {
        if (defined('FOCUS_KEYWORD_FINDER_VERSION')) {
            $this->version = FOCUS_KEYWORD_FINDER_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->focus_keyword_finder = 'focus-keyword-finder';
        $this->name = 'Focus Keyword Finder';
        $this->access_key_header_variable = 'X-FocusKeywordFinder-Access-Key';
        $this->base_url = 'https://focuskeywordfinder.com';
        $this->plugin_update_url = $this->base_url . '/plugin/update.json';
        // $this->plugin_url = $this->base_url . '/plugin/focus-keyword-finder.zip';
        $this->route = $this->focus_keyword_finder.'/v1';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
    }

    private function load_dependencies()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-focus-keyword-finder-loader.php';

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-focus-keyword-finder-i18n.php';

        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-focus-keyword-finder-admin.php';

        $this->loader = new Focus_Keyword_Finder_Loader();
    }

    private function set_locale()
    {
        $plugin_i18n = new Focus_Keyword_Finder_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    private function define_admin_hooks()
    {
        $plugin_admin = new Focus_Keyword_Finder_Admin($this->get_focus_keyword_finder(), $this->get_version(), $this->get_name(), $this->get_route(), $this->get_access_key_header_variable(), $this->get_plugin_update_url(), $this->get_base_url());

        // Actions
        $this->loader->add_action('admin_menu', $plugin_admin, 'admin_menu');
        $this->loader->add_action('admin_init', $plugin_admin, 'admin_init');
        $this->loader->add_action('rest_api_init', $plugin_admin, 'rest_api_init');

        // Filters
        $this->loader->add_filter('wp_kses_allowed_html', $plugin_admin, 'allow_specific_html_tags', 10, 2);
        $this->loader->add_filter('plugin_row_meta', $plugin_admin, 'plugin_row_meta', 10, 2);
        $this->loader->add_filter('update_plugins_focuskeywordfinder.com', 'update_plugin', 10, 2);
    }

    public function run()
    {
        $this->loader->run();
    }

    public function get_focus_keyword_finder()
    {
        return $this->focus_keyword_finder;
    }

    public function get_loader()
    {
        return $this->loader;
    }

    public function get_version()
    {
        return $this->version;
    }

    public function get_name()
    {
        return $this->name;
    }

    public function get_base_url()
    {
        return $this->base_url;
    }

    public function get_plugin_update_url()
    {
        return $this->plugin_update_url;
    }

    public function get_route()
    {
        return $this->route;
    }

    public function get_access_key_header_variable()
    {
        return $this->access_key_header_variable;
    }
}
